@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2020
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,13 +15,13 @@
 --------------------------------------------------------------------------------
 */
 
-//#include "polymake/polytope/ppl_interface_impl.h"
 #include "polymake/polytope/ppl_interface_impl.h"
-#include <polymake/Rational.h>
+#include "polymake/Rational.h"
 
 namespace polymake { namespace polytope { namespace ppl_interface {
 
-template class solver<Rational>;
+template class ConvexHullSolver<Rational>;
+template class LP_Solver<Rational>;
 
 // instantiate this only once in the entire extension!
 fp_mode_setter::init fp_mode_setter::captured;
